Index: src/Fl_Value_Input.cxx
===================================================================
--- src/Fl_Value_Input.cxx	(revision 6608)
+++ src/Fl_Value_Input.cxx	(working copy)
@@ -136,6 +136,12 @@
   set_flag(SHORTCUT_LABEL);
 }
 
+Fl_Value_Input::~Fl_Value_Input() {
+
+  if (input.parent() == (Fl_Group *)this)
+    input.parent(0);   // *revert* ctor kludge!
+}
+
 //
 // End of "$Id$".
 //
Index: src/Fl_Group.cxx
===================================================================
--- src/Fl_Group.cxx	(revision 6608)
+++ src/Fl_Group.cxx	(working copy)
@@ -383,21 +383,23 @@
   affects all child widgets and deletes them from memory.
 */
 void Fl_Group::clear() {
-  Fl_Widget*const* old_array = array();
-  int old_children = children();
-  // clear everything now, in case fl_fix_focus recursively calls us:
-  children_ = 0;
-  //array_ = 0; //dont do this, it will clobber old_array if only one child
   savedfocus_ = 0;
   resizable_ = this;
   init_sizes();
   // okay, now it is safe to destroy the children:
-  Fl_Widget*const* a = old_array;
-  for (int i=old_children; i--;) {
-    Fl_Widget* o = *a++;
-    if (o->parent() == this) delete o;
+  while (children_) {
+    Fl_Widget* o = child(0);	// *first* child widget
+    if (o->parent() == this) {	// should always be true
+      remove(o);		// remove child widget first
+      delete o;			// then delete it
+    } else {			// this should never happen !
+#ifdef DEBUG_CLEAR
+      printf ("Fl_Group::clear() widget:%p, parent: %p != this (%p)\n",
+	      o, o->parent(), this); fflush(stdout);
+#endif // DEBUG_CLEAR
+      remove(o);		// remove it
+    }
   }
-  if (old_children > 1) free((void*)old_array);
 }
 
 /**
@@ -466,7 +468,18 @@
   int i = find(o);
   if (i >= children_) return;
   if (&o == savedfocus_) savedfocus_ = 0;
-  o.parent_ = 0;
+  if (o.parent_ == this) {	// this should always be true
+    o.parent_ = 0;
+  } 
+#ifdef DEBUG_REMOVE  
+  else {			// this should never happen !
+    printf ("Fl_Group::remove(): widget %p, parent_ (%p) != this (%p)\n",
+      &o, o.parent_, this);
+  }
+#endif // DEBUG_REMOVE
+
+  // remove the widget from the group
+
   children_--;
   if (children_ == 1) { // go from 2 to 1 child
     Fl_Widget *t = array_[!i];
Index: src/Fl_Widget.cxx
===================================================================
--- src/Fl_Widget.cxx	(revision 6608)
+++ src/Fl_Widget.cxx	(working copy)
@@ -135,14 +135,19 @@
 
 /**
    Destroys the widget, taking care of throwing focus before if any.
-   Destruction does not remove from any parent group!  And groups when
-   destroyed destroy all their children.  This is convenient and fast.
-   However, it is only legal to destroy a "root" such as an Fl_Window,
-   and automatic destructors may be called.
+   Destruction removes the widget from any parent group! And groups when
+   destroyed destroy all their children. This is convenient and fast.
 */
 Fl_Widget::~Fl_Widget() {
   Fl::clear_widget_pointer(this);
   if (flags() & COPIED_LABEL) free((void *)(label_.value));
+  // remove from parent group
+  if (parent_) parent_->remove(this);
+#ifdef DEBUG_DELETE
+  if (parent_) { // this should never happen
+    printf("*** Fl_Widget: parent_->remove(this) failed [%p,%p]\n",parent_,this);
+  }
+#endif // DEBUG_DELETE
   parent_ = 0; // Don't throw focus to a parent widget.
   fl_throw_focus(this);
 }
Index: FL/Fl_Value_Input.H
===================================================================
--- FL/Fl_Value_Input.H	(revision 6608)
+++ FL/Fl_Value_Input.H	(working copy)
@@ -76,6 +76,7 @@
   void draw();
   void resize(int,int,int,int);
   Fl_Value_Input(int x,int y,int w,int h,const char *l=0);
+  ~Fl_Value_Input();
 
   /** See void Fl_Value_Input::soft(char s) */
   void soft(char s) {soft_ = s;}
